"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.clientApiKey = exports.clientAuthHeader = void 0;
const apollo_cache_inmemory_1 = require("apollo-cache-inmemory");
const apollo_client_1 = require("apollo-client");
const apollo_link_http_1 = require("apollo-link-http");
const apollo_link_context_1 = require("apollo-link-context");
const credentialsService_1 = require("../services/credentialsService");
const config_1 = require("../config");
// tslint:disable-next-line: no-var-requires`
global.fetch = require("node-fetch");
const apigeeHttpLink = new apollo_link_http_1.HttpLink({
    uri: (0, config_1.Configuration)().apigeeDBUrl,
});
const hasuraHttpLink = new apollo_link_http_1.HttpLink({
    uri: (0, config_1.Configuration)().hasuraDBUrl,
});
let authLinkAuthHeader = (0, apollo_link_context_1.setContext)((_, { headers }) => {
    // get the authentication token from local storage if it exists
    // return the headers to the context so httpLink can read them
    return {
        headers: Object.assign(Object.assign({}, headers), { authorization: credentialsService_1.CredentialsService.getAuthHeaderValue() }),
    };
});
let authLinkApiKey = (0, apollo_link_context_1.setContext)((_, { headers }) => {
    // get the authentication token from local storage if it exists
    // return the headers to the context so httpLink can read them
    return {
        headers: Object.assign(Object.assign({}, headers), { "x-api-key": credentialsService_1.CredentialsService.getApiKey() }),
    };
});
authLinkAuthHeader = authLinkAuthHeader.concat(hasuraHttpLink);
authLinkApiKey = authLinkApiKey.concat(apigeeHttpLink);
exports.clientAuthHeader = new apollo_client_1.ApolloClient({
    link: authLinkAuthHeader,
    cache: new apollo_cache_inmemory_1.InMemoryCache({
        addTypename: false,
    }),
    defaultOptions: {
        query: {
            fetchPolicy: "network-only",
        },
        watchQuery: {
            fetchPolicy: "network-only",
        },
    },
});
exports.clientApiKey = new apollo_client_1.ApolloClient({
    link: authLinkApiKey,
    cache: new apollo_cache_inmemory_1.InMemoryCache({
        addTypename: false,
    }),
    defaultOptions: {
        query: {
            fetchPolicy: "network-only",
        },
        watchQuery: {
            fetchPolicy: "network-only",
        },
    },
});
