"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ESClient = void 0;
// import { Client } from "@elastic/elasticsearch";
const index_1 = require("../lib/lambda-logger/index");
const config_1 = require("../config");
const aws_1 = require("@opensearch-project/opensearch/aws");
const opensearch_1 = require("@opensearch-project/opensearch");
const aws_sdk_1 = __importDefault(require("aws-sdk")); // V2 SDK.
class ESClient {
    constructor(log = index_1.LibLogger.Instance) {
        const elasticUrl = (0, config_1.Configuration)().elasticSearchUrl;
        this.log = log;
        this.esClient = new opensearch_1.Client(Object.assign(Object.assign({}, (0, aws_1.AwsSigv4Signer)({
            region: 'us-east-1',
            service: 'aoss',
            getCredentials: () => new Promise((resolve, reject) => {
                // Any other method to acquire a new Credentials object can be used.
                aws_sdk_1.default.config.getCredentials((err, credentials) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(credentials);
                    }
                });
            }),
        })), { node: elasticUrl // OpenSearch domain URL
         }));
    }
    searchByQuery(emailIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const batchSize = 10000;
            const totalEmailIds = emailIds.length;
            let processedEmailIds = 0;
            const allUsersWithAppCount = [];
            while (processedEmailIds < totalEmailIds) {
                const batchEmailIds = emailIds.slice(processedEmailIds, processedEmailIds + batchSize);
                const params = {
                    index: "paladin-policy-service-dev-*",
                    body: {
                        _source: ["time", "data.emailId", "data.appName", "data.tenantId"],
                        size: 0,
                        query: {
                            bool: {
                                must: [
                                    {
                                        terms: {
                                            "actor.id.keyword": batchEmailIds,
                                        },
                                    },
                                ],
                            },
                        },
                        aggs: {
                            uniqueEmailIds: {
                                terms: {
                                    field: "data.emailId.keyword",
                                    size: batchSize,
                                },
                                aggs: {
                                    uniqueApps: {
                                        terms: {
                                            field: "data.appName.keyword",
                                            size: batchSize,
                                        },
                                        aggs: {
                                            lastLogin: {
                                                top_hits: {
                                                    _source: {
                                                        includes: [
                                                            "data.appName",
                                                            "time",
                                                            "data.emailId",
                                                            "data.tenantId",
                                                        ],
                                                    },
                                                    size: 1,
                                                    sort: [
                                                        {
                                                            time: {
                                                                order: "desc",
                                                            },
                                                        },
                                                    ],
                                                },
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                };
                this.log.info("Searching with params", params);
                try {
                    const result = yield this.esClient.search(params);
                    this.log.info("search params result", result);
                    const uniqueEmailIds = result.body.aggregations.uniqueEmailIds.buckets;
                    const usersWithAppCount = uniqueEmailIds.map((emailBucket) => {
                        const emailId = emailBucket.key;
                        const appBuckets = emailBucket.uniqueApps.buckets;
                        const appInfo = appBuckets.map((appBucket) => {
                            const appName = appBucket.key;
                            const lastLoginInfo = appBucket.lastLogin.hits.hits[0]._source;
                            const tenantId = lastLoginInfo.data.tenant_id;
                            return {
                                appName,
                                tenantId,
                                lastLoginInfo,
                            };
                        });
                        return {
                            emailId,
                            apps: appInfo,
                        };
                    });
                    allUsersWithAppCount.push(...usersWithAppCount);
                    processedEmailIds += batchSize;
                }
                catch (err) {
                    this.log.error("Error while querying Elasticsearch: " + err);
                    return [];
                }
            }
            return allUsersWithAppCount;
        });
    }
}
exports.ESClient = ESClient;
