"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExtensionService = void 0;
const local_environment_1 = require("../enviroments/local_environment");
const extension_queries_1 = require("../graphqlQueries/extension-queries");
const index_1 = require("../lib/lambda-logger/index");
const lodash_1 = require("lodash");
class ExtensionService {
    constructor() {
        this.log = index_1.LambdaLogger.getInstance("ExtensionService");
    }
    userPolicyExists(emailId) {
        return __awaiter(this, void 0, void 0, function* () {
            let policyInfo = {};
            try {
                policyInfo = yield this.getDataFromQuery(extension_queries_1.USER_POLICY_EXISTS, {
                    emailId,
                });
                this.log.info("UserPolicy Exists: ", policyInfo);
                return policyInfo;
            }
            catch (err) {
                this.log.error("Error while checking userTaxonExists data in DB", err);
                return;
            }
        });
    }
    policyAppResourceExists(policyId, appName, productName) {
        return __awaiter(this, void 0, void 0, function* () {
            let policyResourceInfo = {};
            try {
                policyResourceInfo = yield this.getDataFromQuery(extension_queries_1.POLICY_APP_RESOURCE_EXISTS, {
                    policyId,
                    appName,
                    productName,
                });
                this.log.info("PolicyAppResource Exists: ", policyResourceInfo);
                return policyResourceInfo;
            }
            catch (err) {
                this.log.error("Error while checking userTaxonExists data in DB", err);
                return;
            }
        });
    }
    getUserPolicyTaxonomy(emailId, appName, productName) {
        return __awaiter(this, void 0, void 0, function* () {
            let taxonInfo = {};
            let taxonomyName = "customer-location";
            try {
                taxonInfo = yield this.getDataFromQuery(extension_queries_1.GET_USER_POLICY_TAXONOMY, {
                    emailId,
                    appName,
                    productName,
                    taxonomyName,
                });
                this.log.info("UserTaxon exists: ", taxonInfo);
                return taxonInfo;
            }
            catch (err) {
                this.log.error("Error while checking userTaxonExists data in DB", err);
                return;
            }
        });
    }
    getUserPolicyAllTaxonomy(emailId, appName, productName) {
        return __awaiter(this, void 0, void 0, function* () {
            let taxonInfo = {};
            try {
                taxonInfo = yield this.getDataFromQuery(extension_queries_1.GET_USER_POLICY_ALL_TAXONOMY, {
                    emailId,
                    appName,
                    productName,
                });
                this.log.info("UserPolicyAllTaxonomy exists: ", taxonInfo);
                return taxonInfo;
            }
            catch (err) {
                this.log.error("Error while checking userAllTaxonExists data in DB", err);
                return;
            }
        });
    }
    getProductAllTaxonomy(appProductId) {
        return __awaiter(this, void 0, void 0, function* () {
            let taxonInfo = {};
            try {
                taxonInfo = yield this.getDataFromQuery(extension_queries_1.GET_PRODUCT_ALL_TAXONOMY, {
                    appProductId,
                });
                this.log.info("getProductAllTaxonomy exists: ", taxonInfo);
                return taxonInfo;
            }
            catch (err) {
                this.log.error("Error while checking getProductAllTaxonomy data in DB", err);
                return;
            }
        });
    }
    createUserPolicy(emailId, appName, productName, customerId) {
        return __awaiter(this, void 0, void 0, function* () {
            const emailIds = ["chandru.k-s@hp.com", emailId];
            const userData = yield this.getDataFromQuery(extension_queries_1.GET_USER_IDS, {
                emailIds,
            });
            let creatorId = "";
            let subjectId = "";
            userData.user.forEach((user) => {
                if (user.email_id === "chandru.k-s@hp.com") {
                    creatorId = user.id;
                    subjectId = user.id;
                }
                else if (user.email_id === emailId) {
                    subjectId = user.id;
                }
            });
            const appProductData = yield this.getDataFromQuery(extension_queries_1.GET_APP_PRODUCT_ID, {
                appName,
                productName,
            });
            const productTaxonomyNames = ["customer-location"];
            const productTaxonomyData = yield this.getDataFromQuery(extension_queries_1.GET_PRODUCT_TAXONOMY_ID, {
                productName,
                productTaxonomyNames,
            });
            var taxonValue = [
                {
                    customers: [customerId],
                    location: null,
                },
            ];
            var policyInsertInput = {
                created_by: creatorId,
                approved_by: creatorId,
                is_active: true,
                kind: "user",
                notes: "",
                name: null,
                statements: {
                    data: {
                        app_product_id: appProductData.app_product[0].id,
                        taxon_allow: {
                            data: [
                                {
                                    product_taxonomy_id: productTaxonomyData.product_taxonomy[0].id,
                                    value: taxonValue,
                                },
                            ],
                        },
                    },
                },
                subjects: {
                    data: {
                        subject: subjectId,
                    },
                },
            };
            let result = {};
            try {
                result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.CREATE_USER_POLICY,
                    variables: {
                        policyInsertInput,
                    },
                });
                this.log.info("UserPolicy Created: ", result);
                return result;
            }
            catch (err) {
                this.log.error("Error while creating UserPolicy data in DB", err);
                return;
            }
        });
    }
    AddPolicyResource(policyId, appName, productName, customerId) {
        return __awaiter(this, void 0, void 0, function* () {
            const appProductData = yield this.getDataFromQuery(extension_queries_1.GET_APP_PRODUCT_ID, {
                appName,
                productName,
            });
            const productTaxonomyNames = ["customer-location"];
            const productTaxonomyData = yield this.getDataFromQuery(extension_queries_1.GET_PRODUCT_TAXONOMY_ID, {
                productName,
                productTaxonomyNames,
            });
            var taxonValue = [
                {
                    customers: [customerId],
                    location: null,
                },
            ];
            var policyResourceInsertInput = {
                policy_id: policyId,
                app_product_id: appProductData.app_product[0].id,
                taxon_allow: {
                    data: [
                        {
                            product_taxonomy_id: productTaxonomyData.product_taxonomy[0].id,
                            value: taxonValue,
                        },
                    ],
                },
            };
            let result = {};
            try {
                result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.ADD_POLICY_RESOURCE,
                    variables: {
                        policyResourceInsertInput,
                    },
                });
                this.log.info("Added PolicyResource: ", result);
                return result;
            }
            catch (err) {
                this.log.error("Error while Add PolicyResource data in DB", err);
                return;
            }
        });
    }
    AddPolicyTaxonomy(policyResourceId, productName, value) {
        return __awaiter(this, void 0, void 0, function* () {
            const productTaxonomyNames = ["customer-location"];
            const productTaxonomyData = yield this.getDataFromQuery(extension_queries_1.GET_PRODUCT_TAXONOMY_ID, {
                productName,
                productTaxonomyNames,
            });
            var policyResourceTaxonAllowInsertInput = {
                policy_resource_id: policyResourceId,
                product_taxonomy_id: productTaxonomyData.product_taxonomy[0].id,
                value: value,
            };
            let result = {};
            try {
                result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.ADD_POLICY_TAXONOMY,
                    variables: {
                        policyResourceTaxonAllowInsertInput,
                    },
                });
                this.log.info("Added PolicyTaxonomy: ", result);
                return result;
            }
            catch (err) {
                this.log.error("Error while Add PolicyTaxonomy data in DB", err);
                return;
            }
        });
    }
    InsertPolicyTaxonomy(policyResourceId, productTaxonomyId, value) {
        return __awaiter(this, void 0, void 0, function* () {
            var policyResourceTaxonAllowInsertInput = {
                policy_resource_id: policyResourceId,
                product_taxonomy_id: productTaxonomyId,
                value: value,
            };
            let result = {};
            try {
                result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.ADD_POLICY_TAXONOMY,
                    variables: {
                        policyResourceTaxonAllowInsertInput,
                    },
                });
                this.log.info("Added PolicyTaxonomy: ", result);
                return result;
            }
            catch (err) {
                this.log.error("Error while Add PolicyTaxonomy data in DB", err);
                return;
            }
        });
    }
    UpdatePolicyTaxonomy(policyResourceId, productName, value) {
        return __awaiter(this, void 0, void 0, function* () {
            const productTaxonomyNames = ["customer-location"];
            const productTaxonomyData = yield this.getDataFromQuery(extension_queries_1.GET_PRODUCT_TAXONOMY_ID, {
                productName,
                productTaxonomyNames,
            });
            let result = {};
            try {
                result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.UPDATE_POLICY_TAXONOMY,
                    variables: {
                        policyResourceId: policyResourceId,
                        productTaxonomyId: productTaxonomyData.product_taxonomy[0].id,
                        value: value,
                    },
                });
                this.log.info("Updated PolicyTaxonomy: ", result);
                return result;
            }
            catch (err) {
                this.log.error("Error while Updating PolicyTaxonomy data in DB", err);
                return;
            }
        });
    }
    UpdateAllPolicyTaxonomy(policyResourceId, productTaxonomyData, value) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = {};
            try {
                result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.UPDATE_POLICY_TAXONOMY,
                    variables: {
                        policyResourceId: policyResourceId,
                        productTaxonomyId: productTaxonomyData,
                        value: value,
                    },
                });
                this.log.info("Updated PolicyTaxonomy: ", result);
                return result;
            }
            catch (err) {
                this.log.error("Error while Updating PolicyTaxonomy data in DB", err);
                return;
            }
        });
    }
    getUsers(appName, productName, customerId) {
        return __awaiter(this, void 0, void 0, function* () {
            let usersInfo;
            var users = [];
            try {
                if (customerId !== "*") {
                    this.log.info("Getting Users for Customer " + customerId + " for App " + appName);
                    const productTaxonomyNames = ["customer-location"];
                    usersInfo = yield this.getDataFromQuery(extension_queries_1.GET_APP_TAXON_USERS, {
                        appName,
                        productName,
                        productTaxonomyNames,
                    });
                    var roleInfo = yield this.getDataFromQuery(extension_queries_1.GET_APP_ALL_USERS, {
                        appName,
                        productName,
                    });
                    if (usersInfo.policy_resource_taxon_allow[0]) {
                        usersInfo.policy_resource_taxon_allow.forEach((userData) => {
                            if (userData.value && (0, lodash_1.isArray)(userData.value)) {
                                var customerFound = userData.value.filter(function (userDataValue) {
                                    if (userDataValue.customers != null) {
                                        return userDataValue.customers.includes(Number(customerId));
                                    }
                                });
                                if (customerFound[0]) {
                                    var roleName;
                                    var policyResource;
                                    roleInfo.policy_resource.forEach((resourceElement) => {
                                        resourceElement.policy.subject.group.users.forEach((userElement) => {
                                            if (userData.policyResource.policy.subject.user &&
                                                userElement.user.email_id ===
                                                    userData.policyResource.policy.subject.user.email_id) {
                                                policyResource = resourceElement;
                                                return;
                                            }
                                        });
                                    });
                                    if (policyResource) {
                                        var roleElement = policyResource.taxon_allow.filter(function (taxonElement) {
                                            return taxonElement.taxonomy.name === "role";
                                        });
                                        if (roleElement[0]) {
                                            var roleInfoArrary = roleElement[0].value[0];
                                            if (roleInfoArrary) {
                                                roleName = roleInfoArrary["role-name"][0];
                                            }
                                        }
                                    }
                                    if (userData.policyResource.policy.subject.user) {
                                        users.push({
                                            name: userData.policyResource.policy.subject.user
                                                .display_name,
                                            emailId: userData.policyResource.policy.subject.user.email_id,
                                            role: roleName,
                                        });
                                    }
                                }
                            }
                        });
                    }
                }
                else {
                    this.log.info("Getting all Users for App " + appName);
                    if (appName == "dc-portal") {
                        usersInfo = yield this.getDataFromQuery(extension_queries_1.GET_DC_APP_ALL_USERS, {
                            appName,
                            productName,
                        });
                    }
                    else {
                        usersInfo = yield this.getDataFromQuery(extension_queries_1.GET_APP_ALL_USERS, {
                            appName,
                            productName,
                        });
                    }
                    usersInfo.policy_resource.forEach((info) => {
                        var roleNameElement = info.taxon_allow.filter(function (roleInfo) {
                            return roleInfo.taxonomy.name === "role";
                        });
                        var roleName;
                        if (roleNameElement[0]) {
                            var roleInfoArrary = roleNameElement[0].value[0];
                            if (roleInfoArrary) {
                                roleName = roleInfoArrary["role-name"][0];
                            }
                        }
                        info.policy.subject.group.users.forEach((userData) => {
                            var managerEmailId;
                            if (userData.user.policies) {
                                if (userData.user.policies.policy.statements[0]) {
                                    userData.user.policies.policy.statements.forEach((statementData) => {
                                        if (statementData.taxon_allow[0]) {
                                            managerEmailId = statementData.taxon_allow[0].value;
                                        }
                                    });
                                }
                            }
                            users.push({
                                name: userData.user.display_name,
                                emailId: userData.user.email_id,
                                role: roleName,
                                manager_emailId: managerEmailId,
                            });
                        });
                    });
                }
                return users;
            }
            catch (err) {
                this.log.error("Error while getting DC Users", err);
                return;
            }
        });
    }
    getExtendedUserDetails(appName, productName, customerId) {
        var _a, e_1, _b, _c, _d, e_2, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            let usersInfo;
            try {
                if (customerId === "*") {
                    try {
                        this.log.info("Getting All Users for App");
                        usersInfo = yield this.getDataFromQuery(extension_queries_1.GET_EXTENDED_USERS, {
                            appName,
                            productName,
                        });
                    }
                    catch (err) {
                        this.log.info("graphql err:", err);
                    }
                }
                var userDump = [];
                var groupUsers = [];
                let usersManagerEmailId = new Array();
                try {
                    // Parsing userdump data
                    for (var _g = true, _h = __asyncValues(usersInfo.policy_resource), _j; _j = yield _h.next(), _a = _j.done, !_a;) {
                        _c = _j.value;
                        _g = false;
                        try {
                            const data = _c;
                            let name;
                            let emailId;
                            let appName;
                            let managerEmailId = "";
                            let customers = [];
                            let region = [];
                            let country = [];
                            let userGroups = "";
                            if (data.policy.subject.user) {
                                // if user policy exists, retrive all policy and taxonomy data
                                name = data.policy.subject.user.display_name;
                                emailId = data.policy.subject.user.email_id;
                                appName = data.resource.app.display_name;
                                if (data.taxon_allow && data.taxon_allow[0]) {
                                    let isManagerIdExist = false;
                                    try {
                                        for (var _k = true, _l = (e_2 = void 0, __asyncValues(data.taxon_allow)), _m; _m = yield _l.next(), _d = _m.done, !_d;) {
                                            _f = _m.value;
                                            _k = false;
                                            try {
                                                const taxon = _f;
                                                if (taxon.taxonomy.name === "manager-emailid" && taxon.value) {
                                                    isManagerIdExist = true;
                                                    managerEmailId = taxon.value;
                                                    usersManagerEmailId[emailId] = managerEmailId;
                                                }
                                                if (taxon.taxonomy.name === "customer-location") {
                                                    if (taxon.value && typeof taxon.value !== "string") {
                                                        if (taxon.value && taxon.value[0]) {
                                                            taxon.value.forEach((item) => {
                                                                if (item.customers && item.customers[0]) {
                                                                    item.customers.forEach((customer) => {
                                                                        customers.push(customer);
                                                                    });
                                                                }
                                                                if (item.location && item.location[0]) {
                                                                    item.location.forEach((location) => {
                                                                        if (location.region && location.region[0]) {
                                                                            location.region.forEach((reg) => {
                                                                                region.push(reg);
                                                                            });
                                                                        }
                                                                        if (location.country && location.country[0]) {
                                                                            location.country.forEach((con) => {
                                                                                country.push(con);
                                                                            });
                                                                        }
                                                                    });
                                                                }
                                                            });
                                                        }
                                                    }
                                                }
                                            }
                                            finally {
                                                _k = true;
                                            }
                                        }
                                    }
                                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                                    finally {
                                        try {
                                            if (!_k && !_d && (_e = _l.return)) yield _e.call(_l);
                                        }
                                        finally { if (e_2) throw e_2.error; }
                                    }
                                    if (!isManagerIdExist) {
                                        managerEmailId = usersManagerEmailId[emailId]
                                            ? usersManagerEmailId[emailId]
                                            : "";
                                    }
                                }
                                let user = {
                                    name,
                                    emailId,
                                    managerEmailId,
                                    appName,
                                    customers,
                                    region,
                                    country,
                                    userGroups,
                                };
                                userDump.push(user);
                            }
                            else if (data.policy.subject.group) {
                                // Users present in DC/FO user groups
                                if (data.policy.subject.group.users[0]) {
                                    data.policy.subject.group.users.forEach((item) => {
                                        if (item && item.user) {
                                            name = item.user.display_name;
                                            emailId = item.user.email_id;
                                            appName = data.resource.app.display_name;
                                            let userGroup = data.policy.subject.group.name; // Get user group name
                                            let user = {
                                                name,
                                                emailId,
                                                appName,
                                                userGroup,
                                            };
                                            groupUsers.push(user);
                                        }
                                    });
                                }
                            }
                        }
                        finally {
                            _g = true;
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (!_g && !_a && (_b = _h.return)) yield _b.call(_h);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                // Remove duplicate users | Unique users
                groupUsers = Array.from(new Set(groupUsers.map((s) => s.emailId))).map((key) => {
                    return {
                        name: groupUsers.filter((user) => user.emailId === key)[0].name,
                        emailId: key,
                        appName: [
                            ...new Set(groupUsers
                                .filter((user) => user.emailId === key)
                                .map((next) => next.appName)),
                        ],
                        userGroups: [
                            ...new Set(groupUsers
                                .filter((user) => user.emailId === key)
                                .map((next) => next.userGroup)),
                        ], // Aggregate all user group names
                    };
                });
                // Push group only users into user dump
                groupUsers.forEach((useritem) => {
                    const result = userDump.find(({ emailId }) => emailId === useritem.emailId);
                    if (!result) {
                        userDump.push(useritem);
                    }
                });
                // Add group data into userdump
                userDump.forEach((user) => {
                    var _a;
                    user.userGroups = (_a = groupUsers.filter((item) => item.emailId === user.emailId)[0]) === null || _a === void 0 ? void 0 : _a.userGroups;
                });
                // Remove duplicate users | Unique users
                const result = Array.from(new Set(userDump.map((s) => s.name))).map((key) => {
                    let userEmailId = userDump.filter((user) => user.name === key)[0]
                        .emailId;
                    return {
                        name: key,
                        emailId: userDump.filter((user) => user.name === key)[0].emailId,
                        managerEmailId: usersManagerEmailId[userEmailId],
                        appName: userDump
                            .filter((user) => user.name === key)
                            .map((next) => next.appName)
                            .toString(),
                        customers: userDump
                            .filter((user) => user.name === key)
                            .map((next) => next.customers)
                            .toString(),
                        region: userDump
                            .filter((user) => user.name === key)
                            .map((next) => next.region)
                            .toString(),
                        country: userDump
                            .filter((user) => user.name === key)
                            .map((next) => next.country)
                            .toString(),
                        userGroups: [
                            ...new Set(userDump
                                .filter((user) => user.name === key)
                                .map((next) => next.userGroups)),
                        ].toString(),
                    };
                });
                return result;
            }
            catch (err) {
                this.log.error("Error while getting User Dump", err);
                return;
            }
        });
    }
    createUser(emailId, name) {
        return __awaiter(this, void 0, void 0, function* () {
            const tenantId = "ecbb95bf-b9ed-45ee-8cdf-4cc7d1a7a70c";
            this.log.info("User does not exist. Creating new User");
            let createUserResult = yield this.addUserMutation(emailId, null, emailId, name, emailId);
            if (createUserResult &&
                createUserResult.insert_user &&
                createUserResult.insert_user.returning.length > 0) {
                let resultOfAssociation = yield this.addUserTenantAssociation(createUserResult.insert_user.returning[0].id, tenantId);
                return resultOfAssociation;
            }
        });
    }
    insertUserToGroup(emailId, appName, productName, groupName) {
        return __awaiter(this, void 0, void 0, function* () {
            let groupsInfo = yield this.getDataFromQuery(extension_queries_1.GET_USER_GROUPS, {
                emailId,
                appName,
                productName,
            });
            if (groupsInfo.policy[0] && groupsInfo.policy[0].subject.group) {
                this.log.info("User already belongs to group " +
                    groupsInfo.policy[0].subject.group +
                    ". Not adding to the role");
                return {
                    status: false,
                };
            }
            const createdByUserIdResult = yield this.getCreatedByUserId("chandru.k-s@hp.com");
            let userAndGroupIdResult = yield this.getUserAndGroupId(emailId, groupName);
            if (userAndGroupIdResult.status === false) {
                return userAndGroupIdResult;
            }
            if (Array.isArray(userAndGroupIdResult.user) &&
                Array.isArray(userAndGroupIdResult.group) &&
                userAndGroupIdResult.user.length > 0) {
                if (userAndGroupIdResult.group.length > 0) {
                    let addUserResult = yield this.addUserToGroup(userAndGroupIdResult.user[0].id, userAndGroupIdResult.group[0].id, createdByUserIdResult.user[0].id);
                    if (addUserResult.status === false) {
                        return addUserResult;
                    }
                    if (addUserResult.insert_user_group &&
                        Array.isArray(addUserResult.insert_user_group.returning) &&
                        addUserResult.insert_user_group.returning.length > 0) {
                        delete addUserResult.insert_user_group.returning[0]["__typename"];
                        return addUserResult;
                    }
                }
            }
            else {
                this.log.info("Could not add User to the group");
                return {
                    status: false,
                };
            }
        });
    }
    removeUserFromGroup(emailId, appName, group, callFromV99 = true) {
        return __awaiter(this, void 0, void 0, function* () {
            let query = callFromV99 ? extension_queries_1.FETCH_APP_GROUPS : extension_queries_1.FETCH_ALL_APP_GROUPS;
            const groupsInfo = yield this.getDataFromQuery(query, { appName });
            const checkGroupsExistsResult = yield this.getDataFromQuery(extension_queries_1.CHECK_GROUPS_EXISTS, { group });
            if (checkGroupsExistsResult &&
                !(checkGroupsExistsResult.group.length === group.length)) {
                return {
                    status: false,
                    message: "Group does not exist. ",
                };
            }
            let isValidAppGroup = true;
            if (groupsInfo && groupsInfo.group && groupsInfo.group.length > 0) {
                const appGroups = groupsInfo.group.map((group) => group.name);
                group.forEach((group) => {
                    if (!appGroups.includes(group)) {
                        isValidAppGroup = false;
                    }
                });
            }
            if (isValidAppGroup) {
                if (emailId && group && group.length > 0) {
                    try {
                        const result = yield local_environment_1.clientAuthHeader.mutate({
                            mutation: extension_queries_1.REMOVE_USER_FROM_GROUP,
                            variables: { emailId: [emailId], groupName: group },
                        });
                        this.log.info("Removed user from groups");
                        return {
                            status: true,
                            result,
                            message: "Removed user from group(s). ",
                        };
                    }
                    catch (err) {
                        this.log.info("Error while executing mutation REMOVE_USER_FROM_GROUP " + err);
                        return {
                            status: false,
                            message: "Could not remove User from group",
                        };
                    }
                }
                else {
                    this.log.info("User or group does not exist");
                    return {
                        status: false,
                        message: "User or group not exist",
                    };
                }
            }
            else {
                this.log.info("User not present in the group");
                return {
                    status: false,
                    message: "Group is not part of the given App. ",
                };
            }
        });
    }
    verifyAndAddUserToGroup(userId, appName, group) {
        return __awaiter(this, void 0, void 0, function* () {
            const groupsInfo = yield this.getDataFromQuery(extension_queries_1.FETCH_APP_GROUPS, {
                appName,
            });
            const checkGroupsExistsResult = yield this.getDataFromQuery(extension_queries_1.CHECK_GROUPS_EXISTS, { group });
            let groupIds = [];
            if (checkGroupsExistsResult &&
                !(checkGroupsExistsResult.group.length === group.length)) {
                return {
                    status: false,
                    message: "Group does not exist. ",
                };
            }
            checkGroupsExistsResult.group.map((groupInfo) => {
                groupIds.push(groupInfo.id);
            });
            let isValidAppGroup = true;
            if (groupsInfo && groupsInfo.group && groupsInfo.group.length > 0) {
                const appGroups = groupsInfo.group.map((group) => group.name);
                group.forEach((group) => {
                    if (!appGroups.includes(group)) {
                        isValidAppGroup = false;
                    }
                });
            }
            if (isValidAppGroup) {
                if (userId && group && group.length > 0) {
                    try {
                        const createdByUserIdResult = yield this.getCreatedByUserId("chandru.k-s@hp.com");
                        const result = yield this.addUserToGroup(userId, groupIds, createdByUserIdResult.user[0].id);
                        this.log.info("Added user from groups");
                        return {
                            status: true,
                            result,
                            message: "Added user from group(s). ",
                        };
                    }
                    catch (err) {
                        this.log.info("Error while adding user to the group " + err);
                        return {
                            status: false,
                            message: "Could not add User from group",
                        };
                    }
                }
                else {
                    this.log.info("User or group does not exist");
                    return {
                        status: false,
                        message: "User or group not exist",
                    };
                }
            }
            else {
                this.log.info("User not present in the group");
                return {
                    status: false,
                    message: "Group is not part of the given App. ",
                };
            }
        });
    }
    deletePolicyResource(policyResourceIds) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.DELETE_POLICY_RESOURCE,
                    variables: {
                        policyResourceIds,
                    },
                });
                return result.data;
            }
            catch (err) {
                this.log.info("Error in policyUpdate " + err);
                return undefined;
            }
        });
    }
    disablePolicyResource(policyResourceIds) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.DISABLE_POLICY_RESOURCE,
                    variables: {
                        policyResourceIds,
                    },
                });
                return result.data;
            }
            catch (err) {
                this.log.info("Error in disable policy resource " + err);
                return undefined;
            }
        });
    }
    enablePolicyResource(policyResourceIds) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.ENABLE_POLICY_RESOURCE,
                    variables: {
                        policyResourceIds,
                    },
                });
                return result.data;
            }
            catch (err) {
                this.log.info("Error in disable policy resource " + err);
                return undefined;
            }
        });
    }
    getCreatedByUserId(emailId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.info("Inside getCreatedByUserId");
            try {
                let result = yield local_environment_1.clientAuthHeader.query({
                    query: extension_queries_1.GET_CREATED_BY_USER_ID,
                    variables: { emailId: emailId },
                });
                return result.data;
            }
            catch (err) {
                this.log.info("Error while getting createdBy User Id " + err);
                return undefined;
            }
        });
    }
    appPolicyResourceExists(policyId, appName) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientAuthHeader.query({
                    query: extension_queries_1.FETCH_APP_POLICY_RESOURCE,
                    variables: { policyId, appName },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while running checkGroupsExists query " + err);
                return {
                    status: false,
                };
            }
        });
    }
    getPolicyDetails(policyId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_POLICY_DETAILS,
                    variables: { policyId },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting policy details " + err);
                return undefined;
            }
        });
    }
    getPolicyByResourceId(resourceId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_POLICY_RESOURCE,
                    variables: { resourceId },
                });
                return result.data.policy_resource[0].policy_id;
            }
            catch (err) {
                this.log.error("Error getPolicyByResourceId " + err);
                return undefined;
            }
        });
    }
    getProductIdbyPermissionId(permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_PRODUCT_ID_BY_PERMISSION_ID,
                    variables: { id: permissionId },
                });
                return result.data.product_permission[0].product_id;
            }
            catch (err) {
                this.log.error("Error getProductIdbyPermissionId" + err);
                return undefined;
            }
        });
    }
    getUserEmailId(updatedBy) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_USER_EMAIL,
                    variables: { updatedBy },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting User Email " + err);
                return undefined;
            }
        });
    }
    getEntityNameById(entityId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_ENTITY_NAME,
                    variables: { entityId },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting entity name by entity Id " + err);
                return undefined;
            }
        });
    }
    getEntityNameBySubjectId(subjectId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_ENTITY_NAME_BY_POLICY_SUBJECT,
                    variables: { subjectId },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting entity name by policy subject Id  " + err);
                return undefined;
            }
        });
    }
    createAuditLog(entityType, entityId, newData, updatedBy, updatedOn, traceId, entityName) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.log.info("Inserting audit log: entityType: " +
                    entityType +
                    " entityId: " +
                    entityId +
                    " newData: " +
                    newData +
                    " updatedBy: " +
                    updatedBy +
                    " updatedOn: " +
                    updatedOn +
                    " traceId: " +
                    traceId +
                    " entityName " +
                    entityName);
                if (!entityName) {
                    this.log.info("EntityName missing - Entity delete mutation");
                    return undefined;
                }
                const result = yield local_environment_1.clientApiKey.mutate({
                    mutation: extension_queries_1.CREATE_AUDIT_LOG,
                    variables: {
                        entityType,
                        entityId,
                        newData,
                        updatedBy,
                        updatedOn,
                        traceId,
                        entityName,
                    },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error in AuditLog " + err);
                return undefined;
            }
        });
    }
    getAppDetails(appId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_APP_DETAILS,
                    variables: { appId },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting app details " + err);
                return undefined;
            }
        });
    }
    getAppAttributesByAppName(appName) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_APP_ATTRIBUTES_BY_APP_NAME,
                    variables: { appName },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting app details " + err);
                return undefined;
            }
        });
    }
    serviceDiscoveryDocument(productId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.SERVICE_DISCOVERY_DOCUMENT,
                    variables: { id: productId },
                });
                return result.data;
            }
            catch (err) {
                this.log.info("Error while getting product information " + err);
                return undefined;
            }
        });
    }
    getProductDetails(productId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_PRODUCT_DETAILS,
                    variables: { productId },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting product details " + err);
                return undefined;
            }
        });
    }
    getUserAndGroupId(emailId, groupName) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = yield local_environment_1.clientAuthHeader.query({
                    query: extension_queries_1.GET_USER_AND_GROUP_ID,
                    variables: { emailId: emailId, groupName: groupName },
                });
                return result.data;
            }
            catch (err) {
                this.log.info("Error while running GET_USER_AND_GROUP_ID query " + err);
                return {
                    status: false,
                };
            }
        });
    }
    addUserToGroup(userId, groupIds, createdByUserId) {
        return __awaiter(this, void 0, void 0, function* () {
            let insertUser = [];
            if (Array.isArray(groupIds)) {
                groupIds.map((groupId) => {
                    let insertUserObj = {
                        group_id: groupId,
                        user_id: userId,
                        created_by: createdByUserId,
                    };
                    insertUser.push(insertUserObj);
                });
            }
            else {
                insertUser = {
                    group_id: groupIds,
                    user_id: userId,
                    // created_by: "bedeae58-628c-4e4c-a9e8-d1a98dfc88e8",
                    created_by: createdByUserId,
                };
            }
            try {
                let result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.ADD_USER_TO_GROUP,
                    variables: { insertUser: insertUser },
                });
                return result.data;
            }
            catch (err) {
                this.log.info("Error while executing mutation ADD_USER_TO_GROUP " + err);
                return {
                    status: false,
                };
            }
        });
    }
    addUserTenantAssociation(userId, tenantId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.ADD_USER_TENANT_ASSOCIATION,
                    variables: { userId: userId, tenantId: tenantId },
                });
                return result.data;
            }
            catch (err) {
                this.log.info("Error in addUserTenantAssociation " + err);
                return undefined;
            }
        });
    }
    checkUserExists(emailId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = yield local_environment_1.clientAuthHeader.query({
                    query: extension_queries_1.CHECK_USER,
                    variables: { emailId: emailId },
                });
                return result.data;
            }
            catch (err) {
                this.log.info("Error while checking User Exists " + err);
                return undefined;
            }
        });
    }
    addUserMutation(hpIdUserId, hpIdResponse, userName, displayName, email) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = yield local_environment_1.clientAuthHeader.mutate({
                    mutation: extension_queries_1.ADD_USER,
                    variables: {
                        hpIdUserId: hpIdUserId,
                        userName: userName,
                        displayName: displayName,
                        email: email.toLowerCase(),
                        hpIdResponse: hpIdResponse,
                    },
                });
                return result.data;
            }
            catch (err) {
                this.log.info("Error in addUserMutation " + err);
                return undefined;
            }
        });
    }
    getAppSubjectNameDocument(appProductId, subjectId, isAppProduct) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = yield this.getDataFromQueryByApiKey(extension_queries_1.GET_APP_SUBJECT_NAME_DOCUMENT, {
                    appProductId,
                    subjectId,
                    isAppProduct,
                });
                return result;
            }
            catch (err) {
                this.log.info("Error while running GetAppSubjectNameDocument query " + err);
                return {
                    status: false,
                };
            }
        });
    }
    UpdateRequestedValue(reqId, requested_value) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = {};
            try {
                requested_value = JSON.parse(requested_value);
                result = yield local_environment_1.clientApiKey.mutate({
                    mutation: extension_queries_1.UPDATE_REQUESTED_VALUE_MUTATION,
                    variables: {
                        reqId: reqId,
                        requested_value: requested_value,
                    },
                });
                this.log.info("Updated RequestedValue in request table: ", result);
                return result;
            }
            catch (err) {
                this.log.error("Error while UpdateRequestedValue data in DB", err);
                return;
            }
        });
    }
    autoRejectUpdate(contitionValue, updateValue, query = extension_queries_1.REQUEST_NOTIFICATION) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (updateValue &&
                    updateValue.notification_data &&
                    updateValue.notification_data.count == 1) {
                    query = extension_queries_1.UPDATE_REQUEST;
                }
                const result = yield local_environment_1.clientApiKey.mutate({
                    mutation: query,
                    variables: { contitionValue, updateValue },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while running autoRejectUpdate query " + err);
                return {
                    status: err,
                };
            }
        });
    }
    //Get the request where notification has sent atleast once
    getRequestForAutoReject() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_REQUEST,
                    variables: {},
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while running getRequestForAutoReject query " + err);
                return {
                    status: err,
                };
            }
        });
    }
    getDataFromQuery(query, variables) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientAuthHeader.query({
                    query,
                    variables,
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while querying data from DB", err);
                return;
            }
        });
    }
    getDataFromQueryByApiKey(query, variables) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query,
                    variables,
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while querying data from DB", err);
                return;
            }
        });
    }
    getGroupDetails(entityId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_GROUP_DETAILS,
                    variables: { entityId },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting group details " + err);
                return undefined;
            }
        });
    }
    getPersonaDetails(entityId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_PERSONA_DETAILS,
                    variables: { entityId },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting group details " + err);
                return undefined;
            }
        });
    }
    getUserDetails(entityId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_USER_DETAILS,
                    variables: { entityId },
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting group details " + err);
                return undefined;
            }
        });
    }
    updateRequestComments(requestId, requestUpdateInput) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = {};
            try {
                result = yield local_environment_1.clientApiKey.mutate({
                    mutation: extension_queries_1.UPDATE_REQUEST_COMMENTS,
                    variables: {
                        requestId: requestId,
                        requestUpdateInput: requestUpdateInput,
                    },
                });
                this.log.info("Updated comments in request table: ", result);
                return result;
            }
            catch (err) {
                this.log.error("Error while  updating request comments data in DB", err);
                return;
            }
        });
    }
    getUserInfo(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = yield this.getDataFromQueryByApiKey(extension_queries_1.GET_USER_EMAIL, {
                    updatedBy: userId,
                });
                return result;
            }
            catch (err) {
                this.log.info("Error while running GetAppSubjectNameDocument query " + err);
                return {
                    status: false,
                };
            }
        });
    }
    getAllUser() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_All_USERS,
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting user details " + err);
                return undefined;
            }
        });
    }
    getAllAppsForUser() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_APP_LIST_FOR_USERS,
                });
                return result.data;
            }
            catch (err) {
                this.log.error("Error while getting user details " + err);
                return undefined;
            }
        });
    }
    getProductAssociatedApps(productShortName) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_PRODUCT_ASSOCIATED_APPS,
                    variables: { productShortName: productShortName },
                });
                return result.data;
            }
            catch (error) {
                this.log.error("Error while fetching the apps list", error);
                return undefined;
            }
        });
    }
    getProductDeveloperUsersEmail(productShortName) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query: extension_queries_1.GET_PRODUCT_DEVELOPER_USERS_EMAIL,
                    variables: { productShortName: productShortName },
                });
                return result.data;
            }
            catch (error) {
                this.log.error("Error while fetching the apps list", error);
                return undefined;
            }
        });
    }
}
exports.ExtensionService = ExtensionService;
